'use strict';

const _ = require('lodash');
const Promise = require('bluebird');
const {gunzip} = require('zlib');
const ReleaseProto = require('./../../../_proto/hapi/release/release_pb').Release;



class ReleaseRevision {

    constructor(data){
        this._name = data[ReleaseRevision.commonProps.name];
        this._version = data[ReleaseRevision.commonProps.version];
        this._status = data[ReleaseRevision.commonProps.status];
        this._modifiedAt = data[ReleaseRevision.commonProps.modifiedAt];
        this._createdAt = data[ReleaseRevision.commonProps.createdAt];
        this._rawData = data.dataAsBase64;
    }

    get name(){
        return this._name;
    }

    get status(){
        return this._status;
    }

    get decodedObject(){
        return this._decodedObject;
    }

    get createdAt(){
        return this._createdAt;
    }

    get modifiedAt(){
        return this._modifiedAt;
    }
    
    get version(){
        return this._version;
    }

    get info(){
        return this._info;
    }

    getBasicData(){
        return {
            version: this._version,
            createdAt: this._createdAt,
            modifiedAt: this._modifiedAt,
            status: this._status,
            info: this._info,
            name: this._name
        }
    }

    init() {
        return this._getRevisionDecodedData()
            .then((decodedObject) => {
                this._decodedObject = decodedObject;
                this._info = _.get(decodedObject, 'info.description')
            });
    }

    _getRevisionDecodedData(){
        const data = Buffer.from(this._rawData, 'base64');
        return Promise.fromCallback((cb) => gunzip(data, cb))
            .then((buffer) => ReleaseProto.deserializeBinary(new Uint8Array(Buffer.from(buffer))))
            .then((protoObject) => protoObject.toObject());
    }
}

ReleaseRevision.commonProps = {
    name: 'NAME',
    status: 'STATUS',
    version: 'VERSION',
    modifiedAt: 'MODIFIED_AT',
    createdAt: 'CREATED_AT',
    statusses: {
        deployed: 'DEPLOYED'
    }
};

module.exports = ReleaseRevision;